@@ -17,6 +17,7 @@
 #include <dhcp4/json_config_parser.h>
 #include <dhcp4/parser_context.h>
 #include <dhcpsrv/cfg_db_access.h>
+#include <dhcpsrv/cfg_multi_threading.h>
 #include <dhcpsrv/cfgmgr.h>
 #include <dhcpsrv/db_type.h>
 #include <hooks/hooks.h>
@@ -169,18 +170,6 @@ ControlledDhcpv4Srv::loadConfigFile(const std::string& file_name) {
                       "processCommand(\"config-set\", json)");
         }
 
-        bool enabled = false;
-        if (srv_thread_count >= 0) {
-            enabled = true;
-        }
-        if (enabled) {
-            CfgMgr::instance().getCurrentCfg()->setServerThreadCount(srv_thread_count);
-            CfgMgr::instance().getCurrentCfg()->setServerMaxThreadQueueSize(4);
-            LOG_FATAL(dhcp4_logger, DHCP4_MULTI_THREADING_WARNING);
-        }
-        MultiThreadingMgr::instance().apply(enabled,
-            CfgMgr::instance().getCurrentCfg()->getServerThreadCount());
-
         // Now check is the returned result is successful (rcode=0) or not
         // (see @ref isc::config::parseAnswer).
         int rcode;
@@ -203,8 +192,8 @@ ControlledDhcpv4Srv::loadConfigFile(const std::string& file_name) {
 
     LOG_WARN(dhcp4_logger, DHCP4_MULTI_THREADING_INFO)
         .arg(MultiThreadingMgr::instance().getMode() ? "yes" : "no")
-        .arg(MultiThreadingMgr::instance().getPktThreadPoolSize())
-        .arg(CfgMgr::instance().getCurrentCfg()->getServerMaxThreadQueueSize());
+        .arg(MultiThreadingMgr::instance().getThreadPoolSize())
+        .arg(MultiThreadingMgr::instance().getPacketQueueSize());
 
     return (result);
 }
@@ -636,6 +625,31 @@ ControlledDhcpv4Srv::commandStatusGetHandler(const string&,
     return (createAnswer(0, status));
 }
 
+ConstElementPtr
+ControlledDhcpv4Srv::commandStatisticSetMaxSampleCountAllHandler(const string&,
+                                                                 ConstElementPtr args) {
+    StatsMgr& stats_mgr = StatsMgr::instance();
+    ConstElementPtr answer = stats_mgr.statisticSetMaxSampleCountAllHandler(args);
+    // Update the default parameter.
+    long max_samples = stats_mgr.getMaxSampleCountDefault();
+    CfgMgr::instance().getCurrentCfg()->addConfiguredGlobal(
+        "statistic-default-sample-count", Element::create(max_samples));
+    return (answer);
+}
+
+ConstElementPtr
+ControlledDhcpv4Srv::commandStatisticSetMaxSampleAgeAllHandler(const string&,
+                                                               ConstElementPtr args) {
+    StatsMgr& stats_mgr = StatsMgr::instance();
+    ConstElementPtr answer = stats_mgr.statisticSetMaxSampleAgeAllHandler(args);
+    // Update the default parameter.
+    auto duration = stats_mgr.getMaxSampleAgeDefault();
+    long max_age = duration.total_seconds();
+    CfgMgr::instance().getCurrentCfg()->addConfiguredGlobal(
+        "statistic-default-sample-age", Element::create(max_age));
+    return (answer);
+}
+
 ConstElementPtr
 ControlledDhcpv4Srv::processCommand(const string& command,
                                     ConstElementPtr args) {
@@ -863,6 +877,18 @@ ControlledDhcpv4Srv::processConfig(isc::data::ConstElementPtr config) {
         // operation.
     }
 
+    // Configure multi threading
+    try {
+        CfgMultiThreading::apply(CfgMgr::instance().getStagingCfg()->getDHCPMultiThreading());
+        if (MultiThreadingMgr::instance().getMode()) {
+            LOG_FATAL(dhcp4_logger, DHCP4_MULTI_THREADING_WARNING);
+        }
+    } catch (const std::exception& ex) {
+        err << "Error applying multi threading settings: "
+            << ex.what();
+        return (isc::config::createAnswer(CONTROL_RESULT_ERROR, err.str()));
+    }
+
     return (answer);
 }
 
@@ -971,13 +997,13 @@ ControlledDhcpv4Srv::ControlledDhcpv4Srv(uint16_t server_port /*= DHCP4_SERVER_P
         boost::bind(&StatsMgr::statisticSetMaxSampleAgeHandler, _1, _2));
 
     CommandMgr::instance().registerCommand("statistic-sample-age-set-all",
-        boost::bind(&StatsMgr::statisticSetMaxSampleAgeAllHandler, _1, _2));
+        boost::bind(&ControlledDhcpv4Srv::commandStatisticSetMaxSampleAgeAllHandler, this, _1, _2));
 
     CommandMgr::instance().registerCommand("statistic-sample-count-set",
         boost::bind(&StatsMgr::statisticSetMaxSampleCountHandler, _1, _2));
 
     CommandMgr::instance().registerCommand("statistic-sample-count-set-all",
-        boost::bind(&StatsMgr::statisticSetMaxSampleCountAllHandler, _1, _2));
+        boost::bind(&ControlledDhcpv4Srv::commandStatisticSetMaxSampleCountAllHandler, this, _1, _2));
 }
 
 void ControlledDhcpv4Srv::shutdownServer(int exit_value) {
