@@ -425,6 +425,13 @@ public:
     /// of all packets.  Called during reconfigure and shutdown.
     void discardPackets();
 
+    /// @brief Returns value of the test_send_responses_to_source_ flag.
+    ///
+    /// @return value of the test_send_responses_to_source_ flag.
+    bool getSendResponsesToSource() const {
+        return (test_send_responses_to_source_);
+    }
+
 protected:
 
     /// @name Functions filtering and sanity-checking received messages.
@@ -731,6 +738,13 @@ protected:
     /// constructor.
     void setPacketStatisticsDefaults();
 
+    /// @brief Sets value of the test_send_responses_to_source_ flag.
+    ///
+    /// @param value new value of the test_send_responses_to_source_ flag.
+    void setSendResponsesToSource(bool value) {
+        test_send_responses_to_source_ = value;
+    }
+
 public:
 
     /// @brief this is a prefix added to the content of vendor-class option
@@ -907,12 +921,9 @@ protected:
     /// are valid. Make sure that pointers are correct before calling this
     /// function.
     ///
-    /// @note This method is static because it is not dependent on the class
-    /// state.
-    ///
     /// @param ex The exchange holding both the client's message and the
     /// server's response.
-    static void adjustRemoteAddr(Dhcpv4Exchange& ex);
+    void adjustRemoteAddr(Dhcpv4Exchange& ex);
 
     /// @brief converts server-id to text
     /// Converts content of server-id option to a text representation, e.g.
@@ -1060,7 +1071,14 @@ protected:
     /// @brief Controls access to the configuration backends.
     CBControlDHCPv4Ptr cb_control_;
 
+private:
+
+    /// @brief store value that defines if kea will send responses
+    /// to a source address of incoming packet. Only for testing.
+    bool test_send_responses_to_source_;
+
 public:
+
     /// Class methods for DHCPv4-over-DHCPv6 handler
 
     /// @brief Updates statistics for received packets
