@@ -31,6 +31,7 @@
 #include <dhcpsrv/parsers/host_reservation_parser.h>
 #include <dhcpsrv/parsers/host_reservations_list_parser.h>
 #include <dhcpsrv/parsers/ifaces_config_parser.h>
+#include <dhcpsrv/parsers/multi_threading_config_parser.h>
 #include <dhcpsrv/parsers/option_data_parser.h>
 #include <dhcpsrv/parsers/dhcp_queue_control_parser.h>
 #include <dhcpsrv/parsers/simple_parser6.h>
@@ -40,6 +41,7 @@
 #include <hooks/hooks_parser.h>
 #include <log/logger_support.h>
 #include <process/config_ctl_parser.h>
+
 #include <util/encode/hex.h>
 #include <util/strutil.h>
 
@@ -311,7 +313,6 @@ public:
                         }
                     }
 
-
                     if (iface.empty()) {
                         iface = (*subnet)->getIface();
                         continue;
@@ -349,8 +350,6 @@ public:
 
         }
     }
-
-
 };
 
 } // anonymous namespace
@@ -520,6 +519,12 @@ configureDhcp6Server(Dhcpv6Srv& server, isc::data::ConstElementPtr config_set,
                 continue;
             }
 
+            if (config_pair.first == "multi-threading") {
+                MultiThreadingConfigParser parser;
+                parser.parse(*srv_config, config_pair.second);
+                continue;
+            }
+
             if (config_pair.first == "dhcp-queue-control") {
                 DHCPQueueControlParser parser;
                 srv_config->setDHCPQueueControl(parser.parse(config_pair.second));
@@ -627,12 +632,11 @@ configureDhcp6Server(Dhcpv6Srv& server, isc::data::ConstElementPtr config_set,
             }
 
             if (config_pair.first == "shared-networks") {
-                /// We need to create instance of SharedNetworks4ListParser
+                /// We need to create instance of SharedNetworks6ListParser
                 /// and parse the list of the shared networks into the
-                /// CfgSharedNetworks4 object. One additional step is then to
+                /// CfgSharedNetworks6 object. One additional step is then to
                 /// add subnets from the CfgSharedNetworks6 into CfgSubnets6
                 /// as well.
-
                 SharedNetworks6ListParser parser;
                 CfgSharedNetworks6Ptr cfg = srv_config->getCfgSharedNetworks6();
                 parser.parse(cfg, config_pair.second);
@@ -694,7 +698,10 @@ configureDhcp6Server(Dhcpv6Srv& server, isc::data::ConstElementPtr config_set,
                  (config_pair.first == "ddns-override-client-update") ||
                  (config_pair.first == "ddns-replace-client-name") ||
                  (config_pair.first == "ddns-generated-prefix") ||
-                 (config_pair.first == "ddns-qualifying-suffix")) {
+                 (config_pair.first == "ddns-qualifying-suffix") ||
+                 (config_pair.first == "store-extended-info") ||
+                 (config_pair.first == "statistic-default-sample-count") ||
+                 (config_pair.first == "statistic-default-sample-age")) {
                 CfgMgr::instance().getStagingCfg()->addConfiguredGlobal(config_pair.first,
                                                                         config_pair.second);
                 continue;
@@ -725,7 +732,7 @@ configureDhcp6Server(Dhcpv6Srv& server, isc::data::ConstElementPtr config_set,
         // defined as part of shared networks.
         global_parser.sanityChecks(srv_config, mutable_cfg);
 
-        // Validate D2 client confuguration.
+        // Validate D2 client configuration.
         if (!d2_client_cfg) {
             d2_client_cfg.reset(new D2ClientConfig());
         }
@@ -841,7 +848,5 @@ configureDhcp6Server(Dhcpv6Srv& server, isc::data::ConstElementPtr config_set,
     return (answer);
 }
 
-int srv_thread_count = -1;
-
 }  // namespace dhcp
 }  // namespace isc
