@@ -1087,7 +1087,11 @@ public:
     /// Method prints intermediate statistics for all exchanges.
     /// Statistics includes sent, received and dropped packets
     /// counters.
-    void printIntermediateStats() const {
+    ///
+    /// \param clean_report value to generate easy to parse report.
+    /// \param sep_ string used as separator if clean_report enabled..
+    void
+    printIntermediateStats(bool clean_report, std::string clean_sep) const {
         std::ostringstream stream_sent;
         std::ostringstream stream_rcvd;
         std::ostringstream stream_drops;
@@ -1097,18 +1101,32 @@ public:
              it != exchanges_.end(); ++it) {
 
             if (it != exchanges_.begin()) {
-                sep = "/";
+                if (clean_report) {
+                    sep = clean_sep;
+                } else {
+                    sep = "/";
+                }
             }
             stream_sent << sep << it->second->getSentPacketsNum();
             stream_rcvd << sep << it->second->getRcvdPacketsNum();
             stream_drops << sep << it->second->getDroppedPacketsNum();
             stream_reject << sep << it->second->getRejLeasesNum();
         }
+
+        if (clean_report) {
+        std::cout << stream_sent.str()
+                  << clean_sep << stream_rcvd.str()
+                  << clean_sep << stream_drops.str()
+                  << clean_sep << stream_reject.str()
+                  << std::endl;
+
+        } else {
         std::cout << "sent: " << stream_sent.str()
                   << "; received: " << stream_rcvd.str()
                   << "; drops: " << stream_drops.str()
                   << "; rejected: " << stream_reject.str()
                   << std::endl;
+        }
     }
 
     /// \brief Print timestamps of all packets.
