@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -55,7 +55,7 @@ CSVLeaseFile6::append(const Lease6& lease) {
                 static_cast<int>(lease.prefixlen_));
     row.writeAt(getColumnIndex("fqdn_fwd"), lease.fqdn_fwd_);
     row.writeAt(getColumnIndex("fqdn_rev"), lease.fqdn_rev_);
-    row.writeAt(getColumnIndex("hostname"), lease.hostname_);
+    row.writeAtEscaped(getColumnIndex("hostname"), lease.hostname_);
     if (lease.hwaddr_) {
         // We may not have hardware information
         row.writeAt(getColumnIndex("hwaddr"), lease.hwaddr_->toText(false));
@@ -63,7 +63,7 @@ CSVLeaseFile6::append(const Lease6& lease) {
     row.writeAt(getColumnIndex("state"), lease.state_);
     // User context is optional.
     if (lease.getContext()) {
-        row.writeAt(getColumnIndex("user_context"), lease.getContext()->str());
+        row.writeAtEscaped(getColumnIndex("user_context"), lease.getContext()->str());
     }
     try {
         VersionedCSVFile::append(row);
@@ -227,7 +227,7 @@ CSVLeaseFile6::readFqdnRev(const CSVRow& row) {
 
 std::string
 CSVLeaseFile6::readHostname(const CSVRow& row) {
-    std::string hostname = row.readAt(getColumnIndex("hostname"));
+    std::string hostname = row.readAtEscaped(getColumnIndex("hostname"));
     return (hostname);
 }
 
@@ -264,7 +264,7 @@ CSVLeaseFile6::readState(const util::CSVRow& row) {
 
 ConstElementPtr
 CSVLeaseFile6::readContext(const util::CSVRow& row) {
-    std::string user_context = row.readAt(getColumnIndex("user_context"));
+    std::string user_context = row.readAtEscaped(getColumnIndex("user_context"));
     if (user_context.empty()) {
         return (ConstElementPtr());
     }
