@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2017 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -25,11 +25,25 @@
 #include <time.h>
 
 
+using namespace isc::db;
 using namespace std;
 
 namespace isc {
 namespace dhcp {
 
+LeasePageSize::LeasePageSize(const size_t page_size)
+    : page_size_(page_size) {
+
+    if (page_size_ == 0) {
+        isc_throw(OutOfRange, "page size of retrieved leases must not be 0");
+    }
+
+    if (page_size_ > std::numeric_limits<uint32_t>::max()) {
+        isc_throw(OutOfRange, "page size of retrieved leases must not be greater than "
+                  << std::numeric_limits<uint32_t>::max());
+    }
+}
+
 Lease6Ptr
 LeaseMgr::getLease6(Lease::Type type, const DUID& duid,
                     uint32_t iaid, SubnetID subnet_id) const {
@@ -65,6 +79,11 @@ LeaseMgr::recountLeaseStats4() {
     stats_mgr.setValue("reclaimed-declined-addresses", zero);
     stats_mgr.setValue("reclaimed-leases", zero);
 
+    // Create if it does not exit cumulative global stats.
+    if (!stats_mgr.getObservation("cumulative-assigned-addresses")) {
+        stats_mgr.setValue("cumulative-assigned-addresses", zero);
+    }
+
     // Clear subnet level stats.  This ensures we don't end up with corner
     // cases that leave stale values in place.
     const Subnet4Collection* subnets =
@@ -111,11 +130,54 @@ LeaseMgr::recountLeaseStats4() {
     }
 }
 
+LeaseStatsQuery::LeaseStatsQuery()
+    : first_subnet_id_(0), last_subnet_id_(0), select_mode_(ALL_SUBNETS) {
+}
+
+LeaseStatsQuery::LeaseStatsQuery(const SubnetID& subnet_id)
+    : first_subnet_id_(subnet_id), last_subnet_id_(0),
+    select_mode_(SINGLE_SUBNET) {
+
+    if (first_subnet_id_ == 0) {
+        isc_throw(BadValue, "LeaseStatsQuery: subnet_id_ must be > 0");
+    }
+}
+
+LeaseStatsQuery::LeaseStatsQuery(const SubnetID& first_subnet_id,
+                                 const SubnetID& last_subnet_id)
+    : first_subnet_id_(first_subnet_id), last_subnet_id_(last_subnet_id),
+    select_mode_(SUBNET_RANGE) {
+
+    if (first_subnet_id_ == 0) {
+        isc_throw(BadValue, "LeaseStatsQuery: first_subnet_id_ must be > 0");
+    }
+
+    if (last_subnet_id_ == 0) {
+        isc_throw(BadValue, "LeaseStatsQuery: last_subnet_id_ must be > 0");
+    }
+
+    if (last_subnet_id_ <= first_subnet_id_) {
+        isc_throw(BadValue,
+                  "LeaseStatsQuery: last_subnet_id_must be > first_subnet_id_");
+    }
+}
+
 LeaseStatsQueryPtr
 LeaseMgr::startLeaseStatsQuery4() {
     return(LeaseStatsQueryPtr());
 }
 
+LeaseStatsQueryPtr
+LeaseMgr::startSubnetLeaseStatsQuery4(const SubnetID& /* subnet_id */) {
+    return(LeaseStatsQueryPtr());
+}
+
+LeaseStatsQueryPtr
+LeaseMgr::startSubnetRangeLeaseStatsQuery4(const SubnetID& /* first_subnet_id */,
+                                           const SubnetID& /* last_subnet_id */) {
+    return(LeaseStatsQueryPtr());
+}
+
 bool
 LeaseStatsQuery::getNextRow(LeaseStatsRow& /*row*/) {
     return (false);
@@ -142,6 +204,14 @@ LeaseMgr::recountLeaseStats6() {
     stats_mgr.setValue("reclaimed-declined-addresses", zero);
     stats_mgr.setValue("reclaimed-leases", zero);
 
+    // Create if it does not exit cumulative global stats.
+    if (!stats_mgr.getObservation("cumulative-assigned-nas")) {
+        stats_mgr.setValue("cumulative-assigned-nas", zero);
+    }
+    if (!stats_mgr.getObservation("cumulative-assigned-pds")) {
+        stats_mgr.setValue("cumulative-assigned-pds", zero);
+    }
+
     // Clear subnet level stats.  This ensures we don't end up with corner
     // cases that leave stale values in place.
     const Subnet6Collection* subnets =
@@ -184,7 +254,7 @@ LeaseMgr::recountLeaseStats6() {
                                        generateName("subnet", row.subnet_id_,
                                                     "assigned-nas"),
                                        row.state_count_);
-                } if (row.lease_state_ == Lease::STATE_DECLINED) {
+                } else if (row.lease_state_ == Lease::STATE_DECLINED) {
                     // Set subnet level value.
                     stats_mgr.setValue(StatsMgr::
                                        generateName("subnet", row.subnet_id_,
@@ -202,7 +272,7 @@ LeaseMgr::recountLeaseStats6() {
                     stats_mgr.setValue(StatsMgr::
                                        generateName("subnet", row.subnet_id_,
                                                     "assigned-pds"),
-                                        row.state_count_);
+                                       row.state_count_);
                 }
                 break;
 
@@ -218,6 +288,17 @@ LeaseMgr::startLeaseStatsQuery6() {
     return(LeaseStatsQueryPtr());
 }
 
+LeaseStatsQueryPtr
+LeaseMgr::startSubnetLeaseStatsQuery6(const SubnetID& /* subnet_id */) {
+    return(LeaseStatsQueryPtr());
+}
+
+LeaseStatsQueryPtr
+LeaseMgr::startSubnetRangeLeaseStatsQuery6(const SubnetID& /* first_subnet_id */,
+                                           const SubnetID& /* last_subnet_id */) {
+    return(LeaseStatsQueryPtr());
+}
+
 std::string
 LeaseMgr::getDBVersion() {
     isc_throw(NotImplemented, "LeaseMgr::getDBVersion() called");
