@@ -8,6 +8,7 @@
 
 #include <cc/data.h>
 #include <dhcpsrv/parsers/simple_parser6.h>
+
 #include <boost/foreach.hpp>
 
 using namespace isc::data;
@@ -35,54 +36,58 @@ namespace dhcp {
 /// list and map types for entries.
 /// Order follows global_param rule in bison grammar.
 const SimpleKeywords SimpleParser6::GLOBAL6_PARAMETERS = {
-    { "data-directory",               Element::string },
-    { "preferred-lifetime",           Element::integer },
-    { "min-preferred-lifetime",       Element::integer },
-    { "max-preferred-lifetime",       Element::integer },
-    { "valid-lifetime",               Element::integer },
-    { "min-valid-lifetime",           Element::integer },
-    { "max-valid-lifetime",           Element::integer },
-    { "renew-timer",                  Element::integer },
-    { "rebind-timer",                 Element::integer },
-    { "decline-probation-period",     Element::integer },
-    { "subnet6",                      Element::list },
-    { "shared-networks",              Element::list },
-    { "interfaces-config",            Element::map },
-    { "lease-database",               Element::map },
-    { "hosts-database",               Element::map },
-    { "hosts-databases",              Element::list },
-    { "mac-sources",                  Element::list },
-    { "relay-supplied-options",       Element::list },
-    { "host-reservation-identifiers", Element::list },
-    { "client-classes",               Element::list },
-    { "option-def",                   Element::list },
-    { "option-data",                  Element::list },
-    { "hooks-libraries",              Element::list },
-    { "expired-leases-processing",    Element::map },
-    { "server-id",                    Element::map },
-    { "dhcp4o6-port",                 Element::integer },
-    { "control-socket",               Element::map },
-    { "dhcp-queue-control",           Element::map },
-    { "dhcp-ddns",                    Element::map },
-    { "user-context",                 Element::map },
-    { "comment",                      Element::string },
-    { "sanity-checks",                Element::map },
-    { "reservations",                 Element::list },
-    { "config-control",               Element::map },
-    { "server-tag",                   Element::string },
-    { "reservation-mode",             Element::string },
-    { "calculate-tee-times",          Element::boolean },
-    { "t1-percent",                   Element::real },
-    { "t2-percent",                   Element::real },
-    { "loggers",                      Element::list },
-    { "hostname-char-set",            Element::string },
-    { "hostname-char-replacement",    Element::string },
-    { "ddns-send-updates",            Element::boolean },
-    { "ddns-override-no-update",      Element::boolean },
-    { "ddns-override-client-update",  Element::boolean },
-    { "ddns-replace-client-name",     Element::string },
-    { "ddns-generated-prefix",        Element::string },
-    { "ddns-qualifying-suffix",       Element::string }
+    { "data-directory",                 Element::string },
+    { "preferred-lifetime",             Element::integer },
+    { "min-preferred-lifetime",         Element::integer },
+    { "max-preferred-lifetime",         Element::integer },
+    { "valid-lifetime",                 Element::integer },
+    { "min-valid-lifetime",             Element::integer },
+    { "max-valid-lifetime",             Element::integer },
+    { "renew-timer",                    Element::integer },
+    { "rebind-timer",                   Element::integer },
+    { "decline-probation-period",       Element::integer },
+    { "subnet6",                        Element::list },
+    { "shared-networks",                Element::list },
+    { "interfaces-config",              Element::map },
+    { "lease-database",                 Element::map },
+    { "hosts-database",                 Element::map },
+    { "hosts-databases",                Element::list },
+    { "mac-sources",                    Element::list },
+    { "relay-supplied-options",         Element::list },
+    { "host-reservation-identifiers",   Element::list },
+    { "client-classes",                 Element::list },
+    { "option-def",                     Element::list },
+    { "option-data",                    Element::list },
+    { "hooks-libraries",                Element::list },
+    { "expired-leases-processing",      Element::map },
+    { "server-id",                      Element::map },
+    { "dhcp4o6-port",                   Element::integer },
+    { "control-socket",                 Element::map },
+    { "dhcp-queue-control",             Element::map },
+    { "dhcp-ddns",                      Element::map },
+    { "user-context",                   Element::map },
+    { "comment",                        Element::string },
+    { "sanity-checks",                  Element::map },
+    { "reservations",                   Element::list },
+    { "config-control",                 Element::map },
+    { "server-tag",                     Element::string },
+    { "reservation-mode",               Element::string },
+    { "calculate-tee-times",            Element::boolean },
+    { "t1-percent",                     Element::real },
+    { "t2-percent",                     Element::real },
+    { "loggers",                        Element::list },
+    { "hostname-char-set",              Element::string },
+    { "hostname-char-replacement",      Element::string },
+    { "ddns-send-updates",              Element::boolean },
+    { "ddns-override-no-update",        Element::boolean },
+    { "ddns-override-client-update",    Element::boolean },
+    { "ddns-replace-client-name",       Element::string },
+    { "ddns-generated-prefix",          Element::string },
+    { "ddns-qualifying-suffix",         Element::string },
+    { "store-extended-info",            Element::boolean },
+    { "statistic-default-sample-count", Element::integer },
+    { "statistic-default-sample-age",   Element::integer },
+    { "multi-threading",                Element::map }
 };
 
 /// @brief This table defines default global values for DHCPv6
@@ -105,9 +110,12 @@ const SimpleDefaults SimpleParser6::GLOBAL6_DEFAULTS = {
     { "ddns-override-client-update",    Element::boolean, "false" },
     { "ddns-replace-client-name",       Element::string,  "never" },
     { "ddns-generated-prefix",          Element::string,  "myhost" },
-    { "ddns-qualifying-suffix",         Element::string,   "" },
-    { "hostname-char-set",              Element::string, "[^A-Za-z0-9.-]" },
-    { "hostname-char-replacement",      Element::string, "" }
+    { "ddns-qualifying-suffix",         Element::string,  "" },
+    { "hostname-char-set",              Element::string,  "[^A-Za-z0-9.-]" },
+    { "hostname-char-replacement",      Element::string,  "" },
+    { "store-extended-info",            Element::boolean, "false" },
+    { "statistic-default-sample-count", Element::integer, "20" },
+    { "statistic-default-sample-age",   Element::integer, "0" }
 };
 
 /// @brief This table defines all option definition parameters.
@@ -207,10 +215,16 @@ const SimpleKeywords SimpleParser6::SUBNET6_PARAMETERS = {
     { "ddns-qualifying-suffix",         Element::string },
     { "hostname-char-set",              Element::string },
     { "hostname-char-replacement",      Element::string },
+    { "store-extended-info",            Element::boolean },
     { "metadata",                       Element::map }
 };
 
 /// @brief This table defines default values for each IPv6 subnet.
+///
+/// Note: When updating this array, please also update SHARED_SUBNET6_DEFAULTS
+/// below. In most cases, those two should be kept in sync, except cases
+/// where a parameter can be derived from shared-networks, but is not
+/// defined on global level.
 const SimpleDefaults SimpleParser6::SUBNET6_DEFAULTS = {
     { "id",               Element::integer, "0" }, // 0 means autogenerate
     { "interface",        Element::string,  "" },
@@ -219,7 +233,11 @@ const SimpleDefaults SimpleParser6::SUBNET6_DEFAULTS = {
     { "interface-id",     Element::string,  "" }
 };
 
-/// @brief This table defines default values for each IPv6 shared network.
+/// @brief This table defines default values for each IPv6 subnet that is
+///        part of a shared network
+///
+/// This is mostly the same as @ref SUBNET6_DEFAULTS, except the parameters
+/// that can be derived from shared-network, but cannot from global scope.
 const SimpleDefaults SimpleParser6::SHARED_NETWORK6_DEFAULTS = {
     { "client-class",     Element::string,  "" },
     { "interface",        Element::string,  "" },
@@ -248,7 +266,8 @@ const ParamsList SimpleParser6::INHERIT_TO_SUBNET6 = {
     "max-valid-lifetime",
     "calculate-tee-times",
     "t1-percent",
-    "t2-percent"
+    "t2-percent",
+    "store-extended-info"
 };
 
 /// @brief This table defines all pool parameters.
@@ -322,6 +341,7 @@ const SimpleKeywords SimpleParser6::SHARED_NETWORK6_PARAMETERS = {
     { "ddns-qualifying-suffix",         Element::string },
     { "hostname-char-set",              Element::string },
     { "hostname-char-replacement",      Element::string },
+    { "store-extended-info",            Element::boolean },
     { "metadata",                       Element::map }
 };
 
@@ -335,13 +355,20 @@ const SimpleDefaults SimpleParser6::IFACE6_DEFAULTS = {
     { "re-detect", Element::boolean, "true" }
 };
 
-/// @brief This table defines default values for dhcp-queue-control in DHCPv4.
+/// @brief This table defines default values for dhcp-queue-control in DHCPv6.
 const SimpleDefaults SimpleParser6::DHCP_QUEUE_CONTROL6_DEFAULTS = {
     { "enable-queue",   Element::boolean, "false"},
     { "queue-type",     Element::string,  "kea-ring6"},
     { "capacity",       Element::integer, "500"}
 };
 
+/// @brief This table defines default values for multi-threading in DHCPv6.
+const SimpleDefaults SimpleParser6::DHCP_MULTI_THREADING6_DEFAULTS = {
+    { "enable-multi-threading", Element::boolean, "false" },
+    { "thread-pool-size",       Element::integer, "0" },
+    { "packet-queue-size",      Element::integer, "64" }
+};
+
 /// @brief This defines default values for sanity checking for DHCPv6.
 const SimpleDefaults SimpleParser6::SANITY_CHECKS6_DEFAULTS = {
     { "lease-checks", Element::string, "warn" }
@@ -415,6 +442,18 @@ size_t SimpleParser6::setAllDefaults(ElementPtr global) {
 
     cnt += setDefaults(mutable_cfg, DHCP_QUEUE_CONTROL6_DEFAULTS);
 
+    // Set the defaults for multi-threading.  If the element isn't there
+    // we'll add it.
+    ConstElementPtr multi_threading = global->get("multi-threading");
+    if (multi_threading) {
+        mutable_cfg = boost::const_pointer_cast<Element>(multi_threading);
+    } else {
+        mutable_cfg = Element::createMap();
+        global->set("multi-threading", mutable_cfg);
+    }
+
+    cnt += setDefaults(mutable_cfg, DHCP_MULTI_THREADING6_DEFAULTS);
+
     // Set the defaults for sanity-checks.  If the element isn't
     // there we'll add it.
     ConstElementPtr sanity_checks = global->get("sanity-checks");
@@ -432,6 +471,7 @@ size_t SimpleParser6::setAllDefaults(ElementPtr global) {
 
 size_t SimpleParser6::deriveParameters(ElementPtr global) {
     size_t cnt = 0;
+
     // Now derive global parameters into subnets.
     ConstElementPtr subnets = global->get("subnet6");
     if (subnets) {
@@ -467,5 +507,5 @@ size_t SimpleParser6::deriveParameters(ElementPtr global) {
     return (cnt);
 }
 
-};
-};
+}  // namespace dhcp
+}  // namespace isc
